package com.persagy.apm.diagnose.config;


import com.google.common.util.concurrent.ThreadFactoryBuilder;
import com.persagy.apm.diagnose.executor.ThreadPoolExecutorExtend;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.scheduling.annotation.EnableAsync;

import java.util.concurrent.*;

/**
 * @author leifeng
 * @version 1.0.0
 * @ClassName ThreadPoolExecutorConfig
 * @Description xia
 * @createTime 2021/9/20 20:58
 */
@Configuration
@EnableAsync
@Slf4j
public class ThreadPoolExecutorConfig {
    /**
     * 获得Java可用的处理器个数 + 1
     */
    private static final int THREADS = Runtime.getRuntime().availableProcessors() + 1;

    @Value("${async.executor.thread.core_pool_size}")
    private int corePoolSize = THREADS;

    @Value("${async.executor.thread.max_pool_size}")
    private int maxPoolSize = 2 * THREADS;

    @Value("${async.executor.thread.queue_capacity}")
    private int queueCapacity = 1024;

    @Value("${async.executor.thread.name.prefix}")
    private String namePrefix = "async-service-";

    final ThreadFactory threadFactory = new ThreadFactoryBuilder()
            // -%d不要少
            .setNameFormat(namePrefix + "%d")
            .setDaemon(true)
            .build();

    /**
     * @throws
     * @title asyncExecutor
     * @description 计算指标线程池
     * @author leifeng
     * @updateTime 2021/9/20 21:26
     * @return: java.util.concurrent.Executor
     */
    @Bean("asyncComputeDataExecutor")
    public Executor asyncExecutor() {
        return new ThreadPoolExecutorExtend(corePoolSize, maxPoolSize,
                5, TimeUnit.SECONDS,
                new LinkedBlockingQueue<>(queueCapacity),
                threadFactory, (r, executor) -> {
            log.info("asyncComputeDataTask is rejected");
        });
    }

    /**
     * @throws
     * @title asyncExecutor
     * @description 计算指标线程池
     * @author leifeng
     * @updateTime 2021/9/20 21:26
     * @return: java.util.concurrent.Executor
     */
    @Bean("asyncSendDataExecutor")
    public Executor asyncSendExecutor() {
        return new ThreadPoolExecutorExtend(corePoolSize, maxPoolSize,
                5, TimeUnit.SECONDS,
                new LinkedBlockingQueue<>(queueCapacity),
                threadFactory, (r, executor) -> {
            log.info("asyncSendDataTask is rejected");
        });
    }
}
