package com.persagy.apm.diagnose.indicatorrecord.controller;

import com.persagy.apm.common.response.ApmResponseUpsertVO;
import com.persagy.apm.common.response.CommonResult;
import com.persagy.apm.common.response.PageList;
import com.persagy.apm.common.utils.ResultHelper;
import com.persagy.apm.diagnose.indicatorrecord.model.ConvertMonitorIndicatorRecordTool;
import com.persagy.apm.diagnose.indicatorrecord.model.dto.*;
import com.persagy.apm.diagnose.indicatorrecord.model.vo.MonitorIndicatorRecordItemVO;
import com.persagy.apm.diagnose.indicatorrecord.model.vo.MonitorIndicatorRecordListItemVO;
import com.persagy.apm.diagnose.indicatorrecord.service.IMonitorIndicatorRecordService;
import com.persagy.framework.ems.data.pojo.hbase.MonitorIndicatorRecord;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.util.List;

/**
 * 监测指标历史记录控制层
 * @author lixing
 * @version V1.0 2021-09-10 00:13:26
 */
@Api(tags = "监测指标历史记录") 
@Validated
@RestController
@RequestMapping("monitorIndicatorRecords")
public class MonitorIndicatorRecordController {
    @Autowired
    private IMonitorIndicatorRecordService monitorIndicatorRecordService;



    @ApiOperation(value = "监测指标历史记录列表")
    @PostMapping("/query")
    public CommonResult<PageList<MonitorIndicatorRecordListItemVO>> queryMonitorIndicatorRecord(
            @Valid @RequestBody QueryMonitorIndicatorRecordDTO queryMonitorIndicatorRecordDTO) throws Exception {
        List<MonitorIndicatorRecord> monitorIndicatorRecordList = monitorIndicatorRecordService.queryMonitorIndicatorRecordList(queryMonitorIndicatorRecordDTO);
        List<MonitorIndicatorRecordListItemVO> resultList = ConvertMonitorIndicatorRecordTool.INSTANCE.convert2List(monitorIndicatorRecordList);
        return ResultHelper.multi(resultList);
    }
    
    @ApiOperation(value = "清除报警定义缓存")
    @PostMapping("/cleanAlarmConfig")
    public CommonResult cleanAlarmConfig(@Valid @RequestBody CleanAlarmConfigDTO CleanAlarmConfigDTO) {
        monitorIndicatorRecordService.cleanAlarmConfig(CleanAlarmConfigDTO.getProjectId());
        return ResultHelper.success();
    }

    @ApiOperation(value = "清除发数时间记录缓存")
    @PostMapping("/cleanAlarmConfigSendTime")
    public CommonResult cleanAlarmConfigSendTime(@Valid @RequestBody CleanAlarmConfigDTO cleanAlarmConfigDTO) {
        monitorIndicatorRecordService.cleanAlarmConfigSendTime(cleanAlarmConfigDTO);
        return ResultHelper.success();
    }
}

