package com.persagy.apm.diagnose.service;

import com.persagy.apm.diagnose.indicatorcompute.model.ProjectIndicatorCompute;
import com.persagy.apm.diagnose.indicatorcompute.service.IProjectIndicatorComputeService;
import com.persagy.apm.diagnose.indicatorrecord.service.IMonitorIndicatorRecordService;
import com.persagy.apm.diagnose.service.dto.ProjectDTO;
import com.persagy.apm.diagnose.utils.CommonUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.Future;

/**
 * @author leifeng
 * @version 1.0.0
 * @ClassName ComputeIndicatorDataService
 * @Description 计算指标数据
 * @createTime 2021/9/20 20:51
 */
@Slf4j
@Service
public class ComputeIndicatorDataService {

    @Autowired
    private SaasWebServiceImpl saasWebServiceImpl;

    @Autowired
    private IMonitorIndicatorRecordService monitorIndicatorRecordService;


    @Autowired
    private IProjectIndicatorComputeService projectIndicatorComputeService;


    /**
     * @title computeIndicatorData
     * @description 计算指标数据
     * @author leifeng
     * @updateTime 2021/9/20 21:26
     */
    public void computeIndicatorData() throws Exception {
        //查询项目列表
        List<ProjectDTO> projectDTOList = saasWebServiceImpl.queryProjectDTOList(null);
        if (CollectionUtils.isEmpty(projectDTOList)) {
            return;
        }

        Map<String, ProjectIndicatorCompute> projectIdAndProjectCompute = projectIndicatorComputeService.getProjectIdAndProjectCompute();

        List<Future<String>> futureList = new ArrayList<>();
//        CountDownLatch latch = new CountDownLatch(projectDTOList.size());
        CountDownLatch latch = new CountDownLatch(1);
        for (ProjectDTO projectDTO : projectDTOList) {
            if(!projectDTO.getProjectId().equals("Pj1101060001")){
                continue;
            }
            futureList.add(monitorIndicatorRecordService.computeIndicatorData(projectDTO, projectIdAndProjectCompute.get(projectDTO.getProjectId()), latch));
        }
        //等待所有任务执行完
        latch.await();
        futureList.forEach(future -> {
            try {
                log.info(future.get());
            } catch (Exception e) {
                log.error("计算项目指标数据错误:" + CommonUtils.getExceptionStackTrace(e));
            }
        });
    }

    public void sendIndicatorData() throws Exception {
        //查询项目列表
        List<ProjectDTO> projectDTOList = saasWebServiceImpl.queryProjectDTOList(null);
        if (CollectionUtils.isEmpty(projectDTOList)) {
            return;
        }
        List<Future<String>> futureList = new ArrayList<>();
//        CountDownLatch latch = new CountDownLatch(projectDTOList.size());
        CountDownLatch latch = new CountDownLatch(1);
        for (ProjectDTO projectDTO : projectDTOList) {
            if(!projectDTO.getProjectId().equals("Pj1101060001")){
                continue;
            }
            futureList.add(monitorIndicatorRecordService.sendIndicatorData(projectDTO, latch));
        }
        //等待所有任务执行完
        latch.await();
        futureList.forEach(future -> {
            try {
                log.info(future.get());
            } catch (Exception e) {
                log.error("发送指标数据错误:" + CommonUtils.getExceptionStackTrace(e));
            }
        });
    }
}
